!generate a random chain and compute Re
! by D.Jost 08/08/09

subroutine rosenbluth(Z,Re2)

implicit none

include 'global.var'

real*8,dimension(NchainMax)		::Z,Re2 !the average Rosenbluth factor and the average end-to-end squared distance
integer,dimension(NchainMax+1,3)	::chain !the bead position
integer,dimension(3)		::site !current bead position
real*8,dimension(Nbmax,4)			::htable !the hash table
real*8							::r2 !the end-to-end distance
real*8							:: W ! current Rosenbluth factor
integer							::Nfn !number of free neighbors
integer,dimension(6)			::FreeNN !identity of the free neighbor 

integer		::h,error,oc,p,next,NN,randNN
real*8		::randomnumber
!initialize variables

chain=0d0
htable=0
W=1d0
!put the first monomer
h=0
htable(h+1,1)=1

next=2
error=0

! we place all the monomers until the entire chain is growth or until we find a dead alley (error=1)
do while ((next.le.(NchainMax+1)).and.(error.eq.0))

	!for the 6 neighbors, look if the site is free
	Nfn=0
	FreeNN=0
	do NN=1,6
		select case(NN)
			case(1)
				site(1)=chain(next-1,1)+1
				site(2)=chain(next-1,2)
				site(3)=chain(next-1,3)
			case(2)
				site(1)=chain(next-1,1)-1
				site(2)=chain(next-1,2)
				site(3)=chain(next-1,3)
			case(3)
				site(1)=chain(next-1,1)
				site(2)=chain(next-1,2)+1
				site(3)=chain(next-1,3)
			case(4)
				site(1)=chain(next-1,1)
				site(2)=chain(next-1,2)-1
				site(3)=chain(next-1,3)
			case(5)
				site(1)=chain(next-1,1)
				site(2)=chain(next-1,2)
				site(3)=chain(next-1,3)+1
			case(6)
				site(1)=chain(next-1,1)
				site(2)=chain(next-1,2)
				site(3)=chain(next-1,3)-1
		end select
		!verify if the site is occupied
		oc=1
		h=mod(mod(a1*site(1)+a2*site(2)+a3*site(3),Nbmax)+Nbmax,Nbmax)
		p=0
		if (htable(h+1,1).eq.0) then!occupation de sitev
			oc=0
		else
			do while ((htable(mod(h+p,Nbmax)+1,1).ne.0).and.((htable(mod(h+p,Nbmax)+1,2).ne.(site(1))) &
				.or.(htable(mod(h+p,Nbmax)+1,3).ne.(site(2))).or.(htable(mod(h+p,Nbmax)+1,4).ne.(site(3))))) 
					p=p+1
			end do
			if (htable(mod(h+p,Nbmax)+1,1).eq.0) oc=0
		end if
		if (oc.eq.0) then !the site is not occupied
			Nfn=Nfn+1
			FreeNN(Nfn)=NN
		end if
	end do
	
	if (Nfn.eq.0) then
		error=1 ! we have a dead alley
	else
		!choose randomly among the free neighbors
		randNN=FreeNN(int(Nfn*randomnumber())+1)
		
		!update chain and htable
		select case(randNN)
			case(1)
				chain(next,1)=chain(next-1,1)+1
				chain(next,2)=chain(next-1,2)
				chain(next,3)=chain(next-1,3)
			case(2)
				chain(next,1)=chain(next-1,1)-1
				chain(next,2)=chain(next-1,2)
				chain(next,3)=chain(next-1,3)
			case(3)
				chain(next,1)=chain(next-1,1)
				chain(next,2)=chain(next-1,2)+1
				chain(next,3)=chain(next-1,3)
			case(4)
				chain(next,1)=chain(next-1,1)
				chain(next,2)=chain(next-1,2)-1
				chain(next,3)=chain(next-1,3)
			case(5)
				chain(next,1)=chain(next-1,1)
				chain(next,2)=chain(next-1,2)
				chain(next,3)=chain(next-1,3)+1
			case(6)
				chain(next,1)=chain(next-1,1)
				chain(next,2)=chain(next-1,2)
				chain(next,3)=chain(next-1,3)-1
		end select
		
		oc=1
		h=mod(mod(a1*chain(next,1)+a2*chain(next,2)+a3*chain(next,3),Nbmax)+Nbmax,Nbmax)
		p=0
		if (htable(h+1,1).eq.0) then!occupation de sitev
			oc=0
		else
			do while ((htable(mod(h+p,Nbmax)+1,1).ne.0).and.((htable(mod(h+p,Nbmax)+1,2).ne.(site(1))) &
				.or.(htable(mod(h+p,Nbmax)+1,3).ne.(site(2))).or.(htable(mod(h+p,Nbmax)+1,4).ne.(site(3))))) 
					p=p+1
			end do
			if (htable(mod(h+p,Nbmax)+1,1).eq.0) oc=0
		end if
		if (oc.eq.0) then !the site is not occupied
			htable(mod(h+p,Nbmax)+1,1)=next
			htable(mod(h+p,Nbmax)+1,2)=chain(next,1)
			htable(mod(h+p,Nbmax)+1,3)=chain(next,2)
			htable(mod(h+p,Nbmax)+1,4)=chain(next,3)
		else
			write(*,*) 'pb rosenbluth'
			stop
		end if
		
		!update rosenbluth factor 
		W=W*Nfn
		
		!update Hits
		Z(next-1)=Z(next-1)+W
		
		!compute Re2
		r2=(real(chain(next,1)-chain(1,1)))**2d0+(real(chain(next,2)-chain(1,2)))**2d0+(real(chain(next,3)-chain(1,3)))**2d0
		Re2(next-1)=Re2(next-1)+r2*W
	end if
	next=next+1
end do

end subroutine rosenbluth